@@ -6,60 +6,14 @@ from iminuit import cost, Minuit
 import iminuit
 from numba_stats import norm, uniform # faster replacements for scipy.stats functions
 import numpy as np
+from termcolor import colored
+import re
 
 
 #
 # NONONO.... I need to go to chebyshev
 #
 
-def print_errors(m2, chi2dof):
-    """
-    I allow a,b,c,d,...  I have longer decimal places
-    """
-    WID =65
-    print
-    print("_"*WID)
-    zn = ""
-
-    form = "y = "
-    xpol = len(m2.parameters)
-    xpolm = len(m2.parameters)
-
-    if chi2dof>1:    zn = "*"
-    print(f" name              value     error{zn}       error%   remark")
-    print("_"*WID)
-    for key in m2.parameters:
-        #if len(key)==1:
-        #    continue
-
-        err = m2.errors[key]
-        val = m2.values[key]
-        if val<0:val=-val
-
-        if chi2dof>1:
-            err = err * np.sqrt(chi2dof)
-
-        print(f"| {key:7} | {val:12.6f} | {err:9.6f}  |  {100*err/val:6.1f}% |", end="")
-
-        xpol-=1
-        form=f"{form} {val:.6f}{'*x'*xpol}"
-        if xpol!=0: form+=" +"
-
-
-        if key=="area":
-            print(f" {100/np.sqrt(val):5.2f}%  (sqrt)|")
-        elif key=="fwhm":
-            print(f" {100*m2.values['fwhm']/m2.values['channel']:5.2f}%  (reso)|")
-        else:
-            print(f"               |")
-
-
-    print("_"*WID)
-    if chi2dof>1: print(f"*... errors WERE scaled up  {np.sqrt(chi2dof):.1f}x     for chi2={chi2dof:.1f} !")
-    print("\n",form) # FORMULA
-
-
-
 
 def main(x,y,dy, polorder = None):
     print("__________________________________________________")
@@ -67,6 +21,64 @@ def main(x,y,dy, polorder = None):
     #bin1 = x[0]
 
 
+
+
+
+    def print_errors(m2, chi2dof):
+        """
+        I allow a,b,c,d,...  I have longer decimal places
+        """
+        WID =65
+        print
+        print("_"*WID)
+        zn = ""
+
+        form = "  "
+        xpol = len(m2.parameters)
+        xpolm = len(m2.parameters)
+
+        if chi2dof>1:    zn = "*"
+        print(f" name              value     error{zn}       error%   remark")
+        print("_"*WID)
+        for key in m2.parameters:
+            #if len(key)==1:
+            #    continue
+
+            err = m2.errors[key]
+            val = m2.values[key]
+            # if val<0:val=-val #why the hell this?
+
+            if chi2dof>1:
+                err = err * np.sqrt(chi2dof)
+
+            print(f"| {key:7} | {val:12.6f} | {err:9.6f}  |  {100*err/val:6.1f}% |", end="")
+
+            xpol-=1
+            form=f"{form} {val:.6f}{'*x'*xpol}"
+            if xpol!=0: form+=" +"
+
+
+            if key=="area":
+                print(f" {100/np.sqrt(val):5.2f}%  (sqrt)|")
+            elif key=="fwhm":
+                print(f" {100*m2.values['fwhm']/m2.values['channel']:5.2f}%  (reso)|")
+            else:
+                print(f"               |")
+
+
+        print("_"*WID)
+        if chi2dof>1: print(f"*... errors WERE scaled up  {np.sqrt(chi2dof):.1f}x     for chi2={chi2dof:.1f} !")
+        print("\n",form,"\n") # FORMULA
+        form1 = re.sub("x", f"{x[0]}", form)
+        form2 = re.sub("x", f"{x[-1]}", form)
+        print(   eval(form1) ,"=",form1)
+        print(   eval(form2) ,"=",form2)
+
+
+
+
+
+
     def model_chi20(x,   a):
         f = a
         return f
@@ -135,16 +147,17 @@ def main(x,y,dy, polorder = None):
 
 
     # ------ create parameter list on the fly
-    params = [ chr(ord('a')+i) for i in  range(0,polorder+1)]
-    #print(params)
-    params =  [ m2.values[i] for i in params ]
-    #print(params)
+    paramnames = [ chr(ord('a')+i) for i in  range(0,polorder+1)]
+    #paramnames =  list("abce")
+    params =  [ m2.values[i] for i in paramnames ]
+
 
     yf = locals()["model_chi2"+str(polorder)]( x, *params )
-                     #m2.values['a'],
-                     #m2.values['b'] )
 
 
+    xf10 = np.arange( x[0],x[-1], (x[-1]-x[0])/100 )
+    yf10 = locals()["model_chi2"+str(polorder)]( xf10, *params )
+
 
     chi2dof=m2.fval/(len(x) - m2.nfit)
     if False:
@@ -160,6 +173,39 @@ def main(x,y,dy, polorder = None):
     #print(f" ... and all ok   ... {NOError}")
 
 
-    # print(f"i... Chebyshev parameters are not real! they are for shifted X")
+    if not(m2.valid):
+        print( colored('X... ____________FIT SEEMS NOT  VALID___________', 'red') )
+    elif  not(m2.accurate):
+        print( colored('X... ____________VALID BUT NOT ACCURATE___________', 'yellow') )
+    else:
+        print( colored('i...    fit seems OK to me', 'green') )
+
+    res = {}
+    res['yf'] = yf
+    res['yf10'] = yf10
+    res['xf10'] = xf10
+
+    res['chi2dof'] = chi2dof
+
+    res['valid'] = m2.valid
+    res['accurate'] = m2.accurate
+
+    res['noerror'] = True
+    #------------ the last one is OR ALL
+    if not(m2.valid) or not(m2.accurate):
+        res['noerror'] = False
+
+    res['logx']       = False
+    res['logy']       = False
+
+
+    res['range']    = ( x[0], x[-1] )
+
+    for i in paramnames:
+        res[i] = m2.values[i]
+        res[f"d{i}"] = m2.errors[i]
+
+
+
     print("_________________________________________________polall")
-    return yf
+    return res
